package sync

import (
	"encoding/json"
	"fmt"
	"log"
	"os"
	"path"
	"time"
	"sync"

	"srs.epita.fr/fic-server/libfic"
	"srs.epita.fr/fic-server/settings"
)

var oneDeepSync sync.Mutex

func SyncDeep(i Importer) (errs map[string][]string) {
	oneDeepSync.Lock()
	defer oneDeepSync.Unlock()

	errs = map[string][]string{}

	errs["_date"] = []string{fmt.Sprintf("%v", time.Now())}
	errs["_themes"] = SyncThemes(i)

	if themes, err := fic.GetThemes(); err == nil {
		for _, theme := range themes {
			errs[theme.Name] = SyncExercices(i, theme)

			if exercices, err := theme.GetExercices(); err == nil {
				for _, exercice := range exercices {
					errs[theme.Name] = append(errs[theme.Name], SyncExerciceFiles(i, exercice)...)
					errs[theme.Name] = append(errs[theme.Name], SyncExerciceHints(i, exercice)...)

					hasFlags := false
					if i.exists(path.Join(exercice.Path, "flags.txt")) {
						errs[theme.Name] = append(errs[theme.Name], SyncExerciceKeys(i, exercice)...)
						hasFlags = true
					}
					if i.exists(path.Join(exercice.Path, "flags-mcq.txt")) || i.exists(path.Join(exercice.Path, "flags-ucq.txt")) {
						errs[theme.Name] = append(errs[theme.Name], SyncExerciceMCQ(i, exercice)...)
						hasFlags = true
					}
					if !hasFlags {
						errs[theme.Name] = append(errs[theme.Name], fmt.Sprintf("%q: has no flags!", path.Base(exercice.Path)))
					}
				}
			}
		}
	}

	errs["_date"] = append(errs["_date"], fmt.Sprintf("%v", time.Now()))

	errs["_regeneration"] = []string{}

	if fdto, err := os.Create("../htdocs-admin/full_import_report.json"); err == nil {
		defer fdto.Close()

		fdto.Write([]byte("disp("))
		if out, err := json.Marshal(errs); err == nil {
			fdto.Write(out)
		} else {
			errs["_regeneration"] = append(errs["_regeneration"], err.Error())
			log.Println(err)
		}
		fdto.Write([]byte(");"))
	} else {
		errs["_regeneration"] = append(errs["_regeneration"], err.Error())
		log.Println(err)
	}

	if err := settings.ForceRegeneration(); err != nil {
		errs["_regeneration"] = append(errs["_regeneration"], err.Error())
	}

	return
}
